@@ -106,5 +106,5 @@ for (est in estimates) {
   lavaanEst[is.na(lavaanEst)] <- -999
   modsemEst <- lavaan::parameterEstimates(est$modsem$lavaan)
   modsemEst[is.na(modsemEst)] <- -999
-  stopifnot(all(lavaanEst == modsemEst))
+  testthat::expect_equal(lavaanEst, modsemEst)
 }
